// This file has been prepared for Doxygen automatic documentation generation.
/*! \file ********************************************************************
*
* Atmel Corporation
*
* - File              : main.c
* - Compiler          : IAR EWAVR 4.11a
*
* - Support mail      : avr@atmel.com
*
* - Supported devices : ATtiny13
*
* - AppNote           : AVR440 - Sensorless control of two-phase brushless
*                       DC motor
*
* - Description       : The full implementation is included in this file.
*
* $Revision: 1.4 $
* $Date: Thursday, September 29, 2005 13:53:48 UTC $
*****************************************************************************/

/*! \mainpage
* \section Intro Introduction
* This and related html pages documents data structures, functions, variables,
* defines, enums, and typedefs used in the software for application note AVR440.\n
*
* \section CI Compilation Info
* This software was written for the IAR Embedded Workbench 4.11B.\n
* \n
* To built the project:\n
* - Add the .c files to project (main.c).
* - Use device --cpu=tiny128,
* - Enable bit definitions in I/O include files
* - lock registers 15 through 9 for global variables.
* - Reduce R-stack to 10.\n
*/

#include <inavr.h>
#include <ioavr.h>
#include "motor.h"
#include "tables.h"

//! The TCCR0A value to be used at next commutation
__no_init __regvar volatile unsigned char nextTCCR0A        @15;

//! The ADMUX value to be used at next commutation
__no_init __regvar volatile unsigned char nextADMUX         @14;

//! The currently active phase.
__no_init __regvar volatile unsigned char activePhase       @13;

//! The next phase to activate.
__no_init __regvar volatile unsigned char nextPhase         @12;

//! Commutation tick counter
__no_init __regvar volatile unsigned int comTick            @10;

//! Flag that tells main loop that speed reference is updated.
__no_init __regvar volatile unsigned char speedRefUpdated   @9;

//! Speed reference.
volatile unsigned char speedRef = 0;

//! The number of ticks between the two last commutations.
volatile unsigned int currentSpeedTick;

//! Number of Watchdog resets. Stored in EEPROM for health monitoring.
__eeprom volatile unsigned int numWatchdogResets;

/*! \brief Program entry point.
 *
 *  Main initializes all subsystems, starts the motor and goes into an
 *  eternal loop where the motor speed is controlled by adjusting the
 *  driving PWM frequency.
 */
void main( void )
{
  unsigned char duty;

  CheckResetSource();
  WDTOff();

  // Initialize all subsystems
  InitPORTB();
  InitPWM();
  InitADC();

  // Run startup procedure
  StartMotor();

  InitWDT();

  __enable_interrupt();

  for(;;)
  {
    duty = CalculateDutyCycle(duty);
    SET_DUTY_BOTH_PHASES(duty);

    WaitMillisecs(1);
  }
}


/*! \brief Examines the source of the last reset and clears the reset flags.
 *
 *  This function polls the MCUSR register for the reset source. Reset sources
 *  that can be identified includes watchdog reset (WDRF), brown-out reset
 *  (BORF), external reset (EXTRF) and power-on reset (PORF).
 *
 *  In this implementation, the flags are just cleared, but it is possible to
 *  take action based on what caused the reset. A watchdog reset typically
 *  indicates that the sensorless commutation failed for some reason. (The
 *  motor was physically blocked, the friction is to large for the fan to
 *  operate properly and so on.) The number of such failures could be stored
 *  to EEPROM to keep track of the health of the motor.
 */
static void CheckResetSource(void)
{
  if (MCUSR & (1 << WDRF))
  {
    // Watchdog was the cause of the reset.
    // Error handling can be done here.
    numWatchdogResets++;
  }
  MCUSR = 0;
}


/*! \brief Initialises PORTB to operate as specified.
 *
 *  This function initializes the PWM control pins as output, every other
 *  pin as input.
 *
 *  If the external tacho signal is enabled, the PORT value of the tacho
 *  is set to 0 for open collector operation.
 */
static void InitPORTB(void)
{
  // Set PWM output pins to output.
  DDRB = (1 << WINDING1_PIN) | (1 << WINDING2_PIN);

  // Initialise tacho pin.
#ifdef TACHO_ENABLED
  PORTB &= (1 << TACHO_OUT_PIN);
#endif
}


/*! \brief  Initializes Timer/Counter1 for PWM operation.
 *
 *  This function initializes Timer/Counter1 for phase correct PWM
 *  operation with OCRA/OCRB as TOP value and starts the PWM (with 0% duty cycle).
 */
static void InitPWM(void)
{
  // Set up TCNT1 for PWM, phase correct mode.
  TCCR0A = PWM_OFF;
  TCCR0B = (0<<WGM02);

  // Start timer
  TCCR0B |= (1 << CS00);
}


/*! \brief Initializes the Analog to Digial converter
 *
 *  This function initializes the Analog to Digital converter.
 */
static void InitADC(void)
{
  // Initialize ADC
  ADCSRA = (1 << ADEN) | (1 << ADATE) | (1 << ADIF) | (0 << ADIE) | ADC_PRESCALER;
  ADCSRB = (1 << ADTS2);
}


/*! \brief Turns off the watchdog timer.
 *
 *  This function turns off the watchdog timer. A watchdog reset will not
 *  be triggered after running this function even if the watchdog timer is
 *  not reset.
 */
static void WDTOff(void)
{
  __disable_interrupt();
  __watchdog_reset();
  WDTCR |= (1 << WDCE) | (1 << WDE);
  WDTCR = 0x00;
  __enable_interrupt();
}


/*! \brief Initializes and turns on the watchdog timer.
 *
 *  This function initialises the watchdog timer and turns it on. After running
 *  this function, the watchdog timer must be periodically reset in order to
 *  avoid a watchdog reset of the chip.
 */
static void InitWDT(void)
{
  __disable_interrupt();
  __watchdog_reset();
  WDTCR |= (1 <<WDCE) | (1 << WDE);
  WDTCR = (1 << WDE) | (0 << WDTIE) | (1 << WDP1) | (0 << WDP1);
  __enable_interrupt();
}


/*! \brief Timer/Counter0 overflow interrupt.
 *
 *  This interrupt service routine is called every time Timer/Counter1 reaches 0.
 *  (Once every PWM cycle). Sensorless commutation and reading of external speed
 *  reference is performed here.
 */
#pragma vector = TIM0_OVF_vect
__interrupt void Timer0_Overflow_ISR(void)
{
  unsigned char adcvalue;
  unsigned char tempADMUX;

  // Chance ADC channel to read analog reference once the zero-cross detection is done.
  tempADMUX = ADMUX;
  ADMUX = ADC_SPEED_REF;

  // Has the necessary time passed since last commutation?
  if (comTick >= ADC_WAIT_TICKS)
  {
    // Wait for AD conversion to complete
    while( (ADCSRA & (1 << ADSC)) )
    {

    }

    // Read ADC value
    adcvalue = ADCH;

    // Commutate if ADC value is lower than commutation threshold
    if (adcvalue < ADC_THRESHOLD)
    {
      TCCR0A = nextTCCR0A;
      tempADMUX = nextADMUX;
      activePhase = nextPhase;

      // Reset watchdog if the motor is not spinning suspicously fast.
      if (comTick > MAX_RPM_TICKS)
      {
        __watchdog_reset();
      }
      // Otherwise, turn off PWM outputs and wait for watchdog reset.
      else
      {
        TCCR0A = PWM_OFF;
        for(;;)
        {
          // Wait here until watchdog reset.
        }
      }

      currentSpeedTick = comTick;
      comTick = 0;


      // Prepare for next commutation and toggle tacho signal
      if (activePhase == PHASE1)
      {
        nextTCCR0A = PWM_PHASE2;
        nextADMUX = ADC_PHASE1;
        nextPhase = PHASE2;
#ifdef TACHO_ENABLED
        DDRB |= (1 << TACHO_OUT_PIN);
#endif
      }
      else
      {
        nextTCCR0A = PWM_PHASE1;
        nextADMUX = ADC_PHASE2;
        nextPhase = PHASE1;
#ifdef TACHO_ENABLED
        DDRB &= ~(1 << TACHO_OUT_PIN);
#endif
      }
    }
    // Start ADC conversion, analog speed reference.
    ADCSRA |= (1 << ADSC);
  }
  else
  {
    // Wait for first AD conversion
    while( (ADCSRA & (1 << ADSC)) )
    {

    }
    // Start ADC conversion, analog speed reference.
    ADCSRA |= (1 << ADSC);
  }

  // Wait for ADC reading (speed ref)
  while( (ADCSRA & (1<<ADSC)) )
  {

  }
  speedRef = ADCH;
  speedRefUpdated = 1;

  // Change ADC channel back for zero-cross detection.
  ADMUX = tempADMUX;

  comTick++;
}


/*! \brief Delay function that uses at least the specified number of milliseconds to complete.
 *
 *  This function takes 1 millisecond to complete if an interrupt does not occur while it is
 *  running. During sensorless operation, the Timer/counter0 overflow interrupt will occur
 *  several times while running this function, increasing the time it takes to return.
 *
 *  \param milliSeconds The number of milliseconds to delay.
 */
static void WaitMillisecs(unsigned int milliSeconds)
{
  unsigned int i;
  for (i = 0; i < milliSeconds; i++)
  {
    __delay_cycles(MILLISEC_NUM_CYCLES);
  }
}


/* \brief Starts the motor according to a predefined sequence.
 *
 *  This function prepositions the rotor and executes a predefined activation
 *  sequence. Sensorless commutation is then activated.
 */
static void StartMotor(void)
{
  unsigned char i;
  unsigned char delay_ms;

  // Lock into phase 1
  TCCR0A = PWM_PHASE1;
  for (i = 0; i < MAX_OCR_VALUE; i++)
  {
    SET_DUTY_PHASE1(i);
    WaitMillisecs(ROTOR_LOCK_RAMPUP_DELAY_MS);
  }

  // Wait for rotor to stabilize
  WaitMillisecs(ROTOR_STABILIZATION_TIME_MS);

  // Let rotor slip to detent position
  for (i = 0; i < MAX_OCR_VALUE; i++)
  {
    SET_DUTY_PHASE1(MAX_OCR_VALUE - i);
    WaitMillisecs(ROTOR_LOCK_RAMPDOWN_DELAY_MS);
  }

  // Disconnect PWM output for a short while
  TCCR0A = PWM_OFF;

  // Set 100% duty cycle on PWM channel A and B
  SET_DUTY_BOTH_PHASES(MAX_OCR_VALUE);

  activePhase = PHASE1;

  i = 0;
  while(delay_ms = startupSequenceDelay[i++])
  {
    if (activePhase == PHASE1)
    {
      TCCR0A = PWM_PHASE2;
      WaitMillisecs(delay_ms);
      activePhase = PHASE2;
    }
    else
    {
      TCCR0A = PWM_PHASE1;
      WaitMillisecs(delay_ms);
      activePhase = PHASE1;
    }
  }

  // initialize all variables needed for sensorless control.
  if (activePhase == PHASE1)
  {
    ADMUX = ADC_PHASE2;
    nextTCCR0A = PWM_PHASE2;
    nextADMUX = ADC_PHASE1;
    nextPhase = PHASE2;
  }
  else
  {
    ADMUX = ADC_PHASE1;
    nextTCCR0A = PWM_PHASE1;
    nextADMUX = ADC_PHASE2;
    nextPhase = PHASE1;
  }

  // Make sure a restart is not triggered.
  currentSpeedTick = (MAX_RPM_TICKS + MIN_RPM_TICKS) / 2;
  comTick = 0;
  speedRefUpdated = 0;

  // Enable timer/counter1 overflow interrupt
  TIMSK0 |= (1 << TOIE0);
}


/*!  Calculates duty cycle based on current speed, speed reference and current duty cycle.
 *
 *  This function calculates a new duty cycle based on the current speed of the rotor,
 *  the external speed reference and the current duty cycle. Any kind of speed controller
 *  could be used here, based on application requirements.
 *
 *  A safety feature that sets max duty cycle if the speed is below a critical limit is
 *  also included in this example. The purpose of this is to ensure that the rotor speed
 *  is high enough for sensorless commutation. Modify this code if there is a possibility
 *  that the current through the motor coils will be too large by doing this.
 */
static unsigned char CalculateDutyCycle(unsigned char duty)
{
  static unsigned int tickSetPoint;
  unsigned long RPMSetPoint;

  // If the speed reference is updated, update the tick setpoint.
  if (speedRefUpdated)
  {
    RPMSetPoint = MAX_RPM * speedRef / 230;
    if (RPMSetPoint < MIN_RPM)
    {
      RPMSetPoint = MIN_RPM;
    }
    tickSetPoint = RPM2COMM_TICKS(RPMSetPoint);
    speedRefUpdated = 0;
  }

  // Very simple speed controller.
  // If current speed too slow -> increase duty cycle.
  // If current speed too fast -> decrease duty cycle.
  if (currentSpeedTick < tickSetPoint)
  {
    if (duty > MIN_OCR_VALUE)
    {
      duty--;
    }
  }
  else
  {
    if (duty < MAX_OCR_VALUE)
    {
      duty++;
    }
    // If speed is lower than critically low speed, maximize duty cycle.
    if (currentSpeedTick > CRITICALLY_LOW_RPM_TICKS)
    {
      duty = MAX_OCR_VALUE;
    }
  }

  return duty;
}
