// This file has been prepared for Doxygen automatic documentation generation.
/*! \file ********************************************************************
*
* Atmel Corporation
*
* - File              : motor.h
* - Compiler          : IAR EWAVR 4.11a
*
* - Support mail      : avr@atmel.com
*
* - Supported devices : ATtiny13
*
* - AppNote           : AVR440 - Sensorless control of two-phase brushless
*                       DC motor
*
* - Description       : All constants (not tables) are included in this file.
*
* $Revision: 1.4 $
* $Date: Thursday, September 29, 2005 13:53:48 UTC $
*****************************************************************************/


#ifndef __MOTOR_H__
#define __MOTOR_H__

/*! \brief System clock frequency.
 *
 *  This is the system clock frequency in Hz. Several parameters depends on the
 *  correct setting of this parameter.
 */
#define SYS_FREQ              9600000UL

/*! \brief Number of clock cycles per second.
 *
 *  The number of clock cycles per second. This is used for the timed sequence in
 *  the motor startup routine.
 */
#define MILLISEC_NUM_CYCLES   (SYS_FREQ / 1000U)

/*! \brief Time to wait for rotor to stabilize after first phase excitation.
 *
 *  This is the time, in milliseconds, to wait for the rotor to stabilize after the
 *  first phase excitation. This parameter will be different for different kinds of
 *  motors.
 */
#define ROTOR_STABILIZATION_TIME_MS       1800U

//! The delay in milliseconds between each increase of duty cycle during startup locking.
#define ROTOR_LOCK_RAMPUP_DELAY_MS        5U

//! The delay in milliseconds between each decrease of duty cycle after startup locking.
#define ROTOR_LOCK_RAMPDOWN_DELAY_MS      1U


/*! \brief The number of clock cycles in 1 "Tick".
 *
 *  This parameter describes the number of clock cycles that passes during 1 "Tick".
 *  (Between two timer overflow interrups.) In phase correct mode, the counter counts
 *  from 0 to 255 to 0, then overflows.
 */
#define TICK_NUM_CYCLES       (510UL)

/*! \brief  Number of commutations per mechanical revolution.
 *
 *  The number of commutations per mechanical revolution.
 */
#define COMMUTATIONS_PER_REV  4UL

/*! \brief The minimum allowed OCR value.
 *
 *  This is the minimum value that can be written to OCR0A/B. This determines the
 *  minimum duty cycle. (Duty cycle = OCR0A/B val / 255).
 */
#define MIN_OCR_VALUE         0x01

/*! \brief The maximum allowed OCR value.
 *
 *  This is the maximum value that can be written to OCR0A/B. This determines the
 *  maximum duty cycle. (Duty cycle = OCR0A/B val / 255). A value of 0xff (255)
 *  equals max. 100% duty cycle.
 */
#define MAX_OCR_VALUE         0xff

/*! \brief Maximum rated speed of the motor.
 *
 *  This parameter specifies the maximum rated speed of the motor.
 */
#define MAX_RPM               2800UL

/*! \brief Minimum speed of the motor.
 *
 *  This parameter specifies the minimum speed the motor should be operated at.
 *  The motor must be able to operate with sensorless commutation at this speed.
 *  It is recommended that a safety margin is put in, such that the motor can
 *  operate with sensorless commutation for even lower speeds.
 */
#define MIN_RPM               550UL

/*! \brief The lowest speed where sensorless commutation can be expected to work reliably.
 *
 *  This parameter specifies the lowest speed where sensorless commutation can be
 *  expected to work reliably. In this implementation, the duty cycle is increased to 100%
 *  if the speed is below this limit.
 */
#define CRITICALLY_LOW_RPM    500UL

/*! \brief Macro that converts speed (RPM) to ticks.
 *
 *  This macro converts a speed given mechanical revolutions per minute (RPM) to the number
 *  of ticks between commutations at this speed.
 *
 *  \param RPM The speed in mechanical revolutions per minute.
 *
 *  \return The number of ticks between commutations at this speed.
 */
#define RPM2COMM_TICKS(RPM)   (unsigned int)(SYS_FREQ / ((RPM * TICK_NUM_CYCLES * COMMUTATIONS_PER_REV) / 60))


//! The number of ticks corresponding to max. speed.
#define MAX_RPM_TICKS             (RPM2COMM_TICKS(MAX_RPM))

//! The number of ticks corresponding to min. speed.
#define MIN_RPM_TICKS             (RPM2COMM_TICKS(MIN_RPM))

//! The number of ticks corresponding to critically low speed.
#define CRITICALLY_LOW_RPM_TICKS  (RPM2COMM_TICKS(CRITICALLY_LOW_RPM))

/*! \brief Pin where phase 1 winding is connected.
 *
 *  This defines the pin where the phase 1 winding is connected. If it is
 *  connected to another pin, change both this value and PWM_PHASE1 must be
 *  changed.
 */
#define WINDING1_PIN             PB0

/*! \brief Pin where phase 1 winding is connected.
 *
 *  This defines the pin where the phase 2 winding is connected. If it is
 *  connected to another pin, change both this value and PWM_PHASE2 must be
 *  changed.
 */
#define WINDING2_PIN             PB1

/*! \brief TCCR0A setting when phase 1 is active.
 *
 *  This parameter specifies the full contents of the TCCR0A register when
 *  phase 1 is active. This should only be changed if winding 1 is connected
 *  to a different pin than PB0.
 *
 */
#define PWM_PHASE1            ((1 << COM0A1) | (0 << COM0A0) | (0 << COM0B1) | (0 << COM0B0) | (0<<WGM01) | (1<<WGM00))

/*! \brief TCCR0A setting when phase 2 is active.
 *
 *  This parameter specifies the full contents of the TCCR0A register when
 *  phase 2 is active. This should only be changed if winding 1 is connected
 *  to a different pin than PB1.
 *
 */
#define PWM_PHASE2            ((0 << COM0A1) | (0 << COM0A0) | (1 << COM0B1) | (0 << COM0B0) | (0<<WGM01) | (1<<WGM00))

/*! \brief TCCR0A setting when no windings are active.
 *
 *  This parameter specifies the full contents of the TCCR0A register when
 *  no windings are active.
 */
#define PWM_OFF               ((0 << COM0A1) | (0 << COM0A0) | (0 << COM0B1) | (0 << COM0B0) | (0<<WGM01) | (1<<WGM00))

/*! \brief ADMUX setting for measurement over phase 1 windings.
 *
 *  This parameter specifies the full contents of the ADMUX register for reading
 *  the voltage over phase 1 windings.
 */
#define ADC_PHASE1            ((1 << REFS0) | (1 << ADLAR) | (1 << MUX1) | (0 << MUX0))

/*! \brief ADMUX setting for measurement over phase 2 windings.
 *
 *  This parameter specifies the full contents of the ADMUX register for reading
 *  the voltage over phase 2 windings.
 */
#define ADC_PHASE2            ((1 << REFS0) | (1 << ADLAR) | (1 << MUX1) | (1 << MUX0))

/*! \brief ADMUX setting for measurement of external speed reference.
 *
 *  This parameter specifies the full contents of the ADMUX register for reading
 *  the external speed reference.
 */
#define ADC_SPEED_REF         ((1 << REFS0) | (1 << ADLAR) | (0 << MUX1) | (1 << MUX0))

//! Selects ADC prescaler
#define ADC_PRESCALER         ((0 << ADPS2) | (1 << ADPS1) | (1 << ADPS0))

//! Phase 1 state variable.
#define PHASE1                0

//! Phase 2 state variable.
#define PHASE2                1

/*! \brief Number of ticks to wait after a commutation before a new zero-crossing can be detected.
 *
 *  This parameter specifies the number of ticks to wait after a commutation, before
 *  the ADC value will be used to detect a new zero-cross.
 */
#define ADC_WAIT_TICKS        50

/*! \brief ADC value of the threshold used to detect a zero-crossing.
 *
 *  This parameter specifies ADC value of the threshold used to detect a zero-crossing.
 *  Adjusting this parameter changes the precise instant of commutation. A higher
 *  threshold generally results in an earlier commutation.
 */
#define ADC_THRESHOLD          0x78

/*! \brief Tacho signal output selection flag.
 *
 *  If this symbol is defined, a tacho signal will be generated at the
 *  TACHO_OUT_PIN (defined later).
 */
//#define TACHO_ENABLED

#ifdef TACHO_ENABLED
/*! \brief Tacho signal output pin.
 *
 *  This parameter specifies which pin will be used to output the tacho signal
 *  (if enabled). If the default tacho output pin (PB5) is used, remember to
 *  disable the external reset.
 */
#define TACHO_OUT_PIN         PB5
#endif

/*! \brief Sets the duty cycle of phase 1
 *
 *  This macro sets the duty cycle of phase 1.
 *
 *  \param duty OCR0A setting.
 */
#define SET_DUTY_PHASE1(duty)         (OCR0A = (duty))

/*! \brief Sets the duty cycle of phase 2
 *
 *  This macro sets the duty cycle of phase 2.
 *
 *  \param duty OCR0B setting.
 */
#define SET_DUTY_PHASE2(duty)         (OCR0B = (duty))

/*! \brief Sets the duty cycle of both phases.
 *
 *  This macro sets the duty cycle of both phases.
 *
 *  \param duty OCR0A/B setting.
 */
#define SET_DUTY_BOTH_PHASES(duty)    (OCR0A = OCR0B = (duty))


// Prototypes
static void CheckResetSource(void);
static void InitPORTB(void);
static void InitPWM(void);
static void InitUART(void);
static void InitADC(void);
static void WDTOff(void);
static void InitWDT(void);
static void WaitMillisecs(unsigned int milliSeconds);
static void StartMotor(void);
static unsigned char CalculateDutyCycle(unsigned char duty);


#endif
